/*******************************************************************************

    ESRMETER_P676(VERSO LCD)


    - FAIXA DE TRABALHO: 0 A 99 OHMS

    - EM 3 ESCALAS OPERANDO DE FORMA AUTOMTICA:

      A)0.00 A 0.99 OHMS
      B) 1 A 9.9
      C) 10 A 99

    - COM PIC 16F676

    AUTOR: CLAUDIO LRIOS

    DATA: 06/05/2022

    IMPORTANTE:
    DESCARREGUE OS CAPACITORES ANTES DE MEDIR SOB RISCO
    DE DANIFICAR O EQUIPAMENTO.
    USE SOMENTE PONTAS DE PROVAS (CABOS) DE BAIXA RESISTNCIA
    ELTRICA (ABAIXO DE 0,5 OHMS). O USO DE PONTAS DE PROVAS
    COM RESISTNCIA ACIMA DE 0,5 OMHS INVIABILIZA A ZERAGEM DO
    APARELHO E IMPEDE MEDIES PRECISAS DA ESCALA DE DEZENAS
    DE MILIOHMS.


    OBS. MONTAGEM  DE NATUREZA DIDTICA

    ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG PICSOURCE


*******************************************************************************/

//=============================================================================/


#include <16F676.h>
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, PUT, NOPROTECT, BROWNOUT, NOMCLR,  NOCPD



//BYTE
#byte portc      = 0x07
#byte porta      = 0x05
#byte trisa      = 0x85
#byte trisc      = 0x87
#byte cmcon      = 0x19
#byte timer0     = 0x01
#byte option_reg = 0x81
#byte t1con      = 0x10
#byte adcon0     = 0x1f
#byte adcon1     = 0x9f
#byte ansel      = 0x91
#byte adresh     = 0x1e
#byte adresl     = 0x9e

//BIT



#bit  escl    = 0x7.5  //escala baixa pino 5 - 100 ohms
#bit  escm    = 0x7.4  //escala mdia pino 6 - 1k
#bit  esch    = 0x5.5  //escala alta  pino 2 - 10k
#bit  desc    = 0x5.4  //descarga do capacitor cx
#bit  sw      = 0x5.3  //boto zerar


#bit   en        = 0x07.1 //rc1  lcd 'enable' pino 9
#bit   rs        = 0x07.2 //rc2  lcd 'rs' pino 8
#bit   db4       = 0x05.0 //ra0  db4 pino 13
#bit   db5       = 0x05.1 //ra1  db5 pino 12
#bit   db6       = 0x05.2 //ra2  db6 pino 11
#bit   db7       = 0x07.0 //rc0  db7 pino 10


#bit  c2      = 0x1f.7 //sada do comparador 1
#bit  c1      = 0x1f.6 //sada do comparador 2
#bit  cmif    = 0x0c.6 //flag interrupo dos comparadores analgicos
#bit  t0if    = 0x0b.2 //flag de interrupo do timer 0
#bit  t1if    = 0x0C.0 //flag do timer1
#bit go_done  = 0x1f.1 //flag de nicio de converso adc


 //variveis globais


 int16 res_esr=0;
 int16 zesrl=0;
 int16 zesrm=0;
 int16 zesrh=0;
 int8 mil;
 int8 cent;
 int8 dez;
 int8 un;
 int1 pt1=0;
 int1 pt2=0;
 int1 f_est=0;//flag de estouro de escalas
 int1 f_zerar=0;//flag de zero
 int8 val;

/******************************************************************************

******************************************************************************/

//==============================================================================
//                     ROTINAS PARA LCD
//==============================================================================
//clock para lcd
void lcd_clk(void){
 en=1;
 delay_us(20);//ajuste conforme o modelo de LCD
 en=0;
 }

//envia nible para lcd
 void env_nib(int8 b){
 if(b & 0b00010000)db4=1; else db4=0;
 if(b & 0b00100000)db5=1; else db5=0;
 if(b & 0b01000000)db6=1; else db6=0;
 if(b & 0b10000000)db7=1; else db7=0;
 lcd_clk();
  }

 //envia comando para lcd
 void env_lcd_c(int8 val){
 rs=0;
 env_nib(val);
 env_nib(swap(val));
 delay_ms(10);
}
//envia dados para lcd
 void env_lcd_d(int8 val){
 rs=1;
 env_nib(val);
 env_nib(swap(val));
 }


//inicializao do lcd
void lcd_init(void){

  //resetar a CGRAM antes de configurar
  //Isto evita no funciona aps ligar/desligar
  //a alimentao do circuito quando pino 4
  //do pic  usado como entrada e no para
  //reset
  delay_ms(100);//40

  rs=0; db4=1; db5=1; db6=1; db7=1;
  lcd_clk();
  lcd_clk();
  lcd_clk();
  rs=0; db4=0; db5=0; db6=0; db7=0;
  lcd_clk();
  lcd_clk();
  lcd_clk();

  delay_ms(100);//40
   rs=0; db4=1; db5=1;db6=0; db7=0;

  lcd_clk();
  delay_ms(5);
  lcd_clk();
  delay_us(160);
  lcd_clk();
  delay_us(160);

  db4=0;
  lcd_clk();
  env_lcd_c(0x28);//ajusta para 4 bits o modo de envio
  env_lcd_c(0x06);// no shift
  env_lcd_c(0x0c);//Display ON, Cursor OFF, Blink OFF
  env_lcd_c(0x01);//limpa tela
  delay_ms(10);//10
}

void delay1seg(){
  delay_ms(1000);
}

/******************************************************************************
           ROTINA DE LEITURA DE ESR: ESCALA L
******************************************************************************/

void leitura_esrl(){ //leitura ESR escala de 0.00 a 0.99 ohms

   pt2=1;//liga ponto das dezenas
   pt1=0;//desliga ponto das unidades
   f_est=0;//zera estouro de escalas
   escl=1;//desliga escalas
   escm=1;
   esch=1;
   res_esr=0;//zera registrador para iniciar leitura
   desc=1; //liga descarga do capacitor sob teste
     delay_ms(32);//aguarda v rampa = 0,6 volts  (10mV/500useg)


     desc=0;//desliga descarga do cap sob teste
     escl=0;//liga escala baixa de esr
     delay_us(10);
     escl=1;//desliga escala baixa de esr
     go_done=1;//liga converso no pino 7 (an7)
     while(go_done);
     res_esr= (int16) adresh<<8;
     res_esr= res_esr  + adresl;
     desc=1;//liga descarga do capacitor sob teste


   if(f_zerar){
   if(res_esr>=zesrl) res_esr= res_esr - zesrl; else res_esr=zesrl - res_esr ;
   }
   delay_us(500);

}

/******************************************************************************
           ROTINA DE LEITURA DE ESR: ESCALA M
******************************************************************************/

void leitura_esrm(){ //leitura ESR escala de 1.0 a 9.9 ohms

   pt2=0;//desliga ponto das dezenas
   pt1=1;//liga ponto das unidades
   escl=1;//desliga escalas
   esch=1;
   escm=1;
   res_esr=0;//zera registrador para iniciar leitura
   desc=1; //liga descarga do capacitor sob teste

   delay_ms(32);//aguarda v rampa = 0,6 volts  (10mV/500useg)


     desc=0;//desliga descarga do cap sob teste
     escm=0;//liga escala mdia de esr
     delay_us(10);
     escm=1;//desliga escala baixa de esr
     go_done=1;//liga converso no pino 7 (an7)

   //  escm=1;//desliga escala baixa de esr
     while(go_done);
     res_esr= (int16) adresh<<8;
     res_esr= res_esr  + adresl;
     desc=1;//liga descarga do capacitor sob teste


   if(f_zerar){
     if(res_esr>=zesrm) res_esr= res_esr- zesrm; else res_esr=zesrm - res_esr;
   }
   delay_us(500);
}

/******************************************************************************
           ROTINA DE LEITURA DE ESR: ESCALA H
******************************************************************************/

void leitura_esrh(){ //leitura ESR escala de 10 a 60 ohms

   pt2=0;//desliga ponto das dezenas
   pt1=0;//liga ponto das unidades
   escl=1;//desliga escalas
   escm=1;
   esch=1;
   res_esr=0;//zera registrador para iniciar leitura
   desc=1; //liga descarga do capacitor sob teste

   delay_ms(32);//aguarda v rampa = 0,6 volts  (10mV/500useg)


     desc=0;//desliga descarga do cap sob teste
     esch=0;//liga escala mdia de esr
     delay_us(10);
     esch=1;//desliga escala baixa de esr
     go_done=1;//liga converso no pino 7 (an7)

   //  escm=1;//desliga escala baixa de esr
     while(go_done);
     res_esr= (int16) adresh<<8;
     res_esr= res_esr  + adresl;
     desc=1;//liga descarga do capacitor sob teste


   if(f_zerar){
     if(res_esr>=zesrh) res_esr= res_esr- zesrh; else res_esr=zesrh - res_esr;
   }
   delay_us(500);
}

//==============================================================================
//         ROTINA MAIN
//==============================================================================

void main() {

   option_reg=0x8F; //timer0 /2 no pullup /watchdog 2,3 seg
   trisa=0b00001000;//entradas (1) e saidas (0)
   trisc=0b00001000;//todos pinos como sadas exceto rc3 (adc in)
   ansel=0b10000000;//canal 7 (an7) rc3 - entrada adc pino 7
   cmcon= 0x07; //desliga comparadores analgicos
   porta=0B00100000;// esch(1)
   portc=0B00110000;//desliga ESCM e ESCL(1)
   lcd_init();//inicializa o lcd
   printf(env_lcd_d," ESRMETER_P676");//mostra nome do aparelho
   env_lcd_c(0xc1);
   printf(env_lcd_d,"by PICSOURCE");//blog
   delay1seg();
   delay1seg();
   env_lcd_c(0x01);//limpar tela
   env_lcd_c(0x84);
   printf(env_lcd_d,"AVISO!");
   env_lcd_c(0xc0);
   printf(env_lcd_d,"Zere antes usar!");
   delay1seg();
   delay1seg();
   env_lcd_c(0x01);//limpar tela
   adcon0= 0b10011101;//ajusta para canal 7 adc e liga conversor adc
   adcon1= 0b00010000; //ajusta para presc/8 tad= 2 useg

//==============================================================================
//         LAO DE REPETIO PRINCIPAL
//==============================================================================

   for(;;){

       leitura_esrl();//leitura na escala de 0,00 a 0,99 ohms
       if(res_esr>99){//se houve estouro nesta escala faz
         leitura_esrm();//leitura na escala de 1 a 9,9 ohms
         if(res_esr>99){//se houve estouro nesta escala faz
           leitura_esrh();
           if(res_esr>99){
           f_est=1;//sinaliza estouro de escalas
           res_esr=99;//evita erros na transformao para 7 seguimentos
           }
         }
      }


     //Zerar se pontas de provas em curto/ boto apertado e flag =0
     if( /*(pt2==1) && */(sw==0) && (f_zerar==0)){//gerar valores para zerar
       env_lcd_c(0x01);//limpar tela
       printf(env_lcd_d,"Zerando:");
       env_lcd_c(0xc0);
       printf(env_lcd_d,"Aguarde...");
       delay1seg();
       leitura_esrl();//leitura na escala de 0,01 a 0,60 ohms
       zesrl=res_esr;//salva valor para zerar desta escala
       leitura_esrm();//leitura na escala de 1 a 6.0 ohms
       zesrm=res_esr;//salva valor para zerar desta escala
       leitura_esrh();//leitura na escala de 10 a 60 ohms
       zesrh=res_esr;//salva valor para zerar desta escala
       f_zerar=1;//seta flag para indicar que foi feita operao de zerar
       env_lcd_c(0x01);//limpar tela
       printf(env_lcd_d,"Zeragem OK!");
       delay1seg();
       env_lcd_c(0x01);//limpar tela


     }



     mil=res_esr/1000 |0x30;
     res_esr=res_esr%1000;
     cent=res_esr/100 |0x30;
     res_esr=res_esr%100;
     dez=res_esr/10 | 0x30;//converte valor encontrado para decimal dezenas
     un= res_esr%10 | 0x30;//converte valor encontrado para decimal unidades

      env_lcd_c(0x8b);
     if(f_zerar == 0){//mostra 'nz' para indicar
       env_lcd_d('n');//que precisa ser zerado
       env_lcd_d('z');
     }
     else{
       env_lcd_d(' ');
       env_lcd_d(' ');
     }

     env_lcd_c(0x86); //envia para linha 1 posio 5
     printf(env_lcd_d,"ESR:");




     env_lcd_c(0xc5); //envia para linha 2 posio 5
     if(pt2){    
       env_lcd_d(cent);
       env_lcd_d('.');
       env_lcd_d(dez);
       env_lcd_d(un);
     }

     else if(pt1){      
       env_lcd_d(dez);
       env_lcd_d('.');//teste
       env_lcd_d(un);

     }

     else{
     if(dez==0x39 && un==0x39){// se igual a '99'
       
       env_lcd_d('>');
       env_lcd_d(' ');
     }
          
       env_lcd_d(dez);
       env_lcd_d(un);
     }

     printf(env_lcd_d," Ohms  ");


     delay_ms(400);

 }//for(;;)

}//main()




